from typing import List, Optional, Union
from pydantic import BaseModel, Field
from dataclasses import dataclass
from producteca.abstract.abstract_dataclass import BaseService
from producteca.products.search_products import SearchProduct, SearchProductParams
import logging
import requests

_logger = logging.getLogger(__name__)


class Attribute(BaseModel):
    key: str
    value: str


class Tag(BaseModel):
    tag: str


class Dimensions(BaseModel):
    weight: Optional[float] = None
    width: Optional[float] = None
    height: Optional[float] = None
    length: Optional[float] = None
    pieces: Optional[int] = None


class Deal(BaseModel):
    campaign: str
    regular_price: Optional[float] = Field(default=None, alias='regularPrice')
    deal_price: Optional[float] = Field(default=None, alias='dealPrice')


class Stock(BaseModel):
    quantity: Optional[int] = None
    available_quantity: Optional[int] = Field(default=None, alias='availableQuantity')
    warehouse: Optional[str] = None
    warehouse_id: Optional[int] = Field(default=None, alias='warehouseId')
    reserved: Optional[int] = None
    available: Optional[int] = None


class Price(BaseModel):
    amount: Optional[float] = None
    currency: str
    price_list: str = Field(alias='priceList')
    price_list_id: Optional[int] = Field(default=None, alias='priceListId')


class Picture(BaseModel):
    url: str


class Integration(BaseModel):
    app: Optional[int] = None
    integration_id: Optional[str] = Field(default=None, alias='integrationId')
    permalink: Optional[str] = None
    status: Optional[str] = None
    listing_type: Optional[str] = Field(default=None, alias='listingType')
    safety_stock: Optional[int] = Field(default=None, alias='safetyStock')
    synchronize_stock: Optional[bool] = Field(default=None, alias='synchronizeStock')
    is_active: Optional[bool] = Field(default=None, alias='isActive')
    is_active_or_paused: Optional[bool] = Field(default=None, alias='isActiveOrPaused')
    id: Optional[int] = None
    parent_integration: Optional[str] = Field(default=None, alias='parentIntegration')


class Variation(BaseModel):
    variation_id: Optional[int] = Field(default=None, alias='variationId')
    components: Optional[List] = None
    pictures: Optional[List[Picture]] = None
    stocks: Optional[List[Stock]] = None
    attributes_hash: Optional[str] = Field(default=None, alias='attributesHash')
    primary_color: Optional[str] = Field(default=None, alias='primaryColor')
    thumbnail: Optional[str] = None
    attributes: Optional[List[Attribute]] = None
    integrations: Optional[List[Integration]] = None
    id: Optional[int] = None
    sku: Optional[str] = None
    barcode: Optional[str] = None


class MeliCategory(BaseModel):
    meli_id: Optional[str] = Field(default=None, alias='meliId')
    accepts_mercadoenvios: Optional[bool] = Field(default=None, alias='acceptsMercadoenvios')
    suggest: Optional[bool] = None
    fixed: Optional[bool] = None


class Product(BaseModel):
    sku: Optional[str] = None
    variation_id: Optional[int] = None
    code: Optional[str] = None
    name: Optional[str] = None
    barcode: Optional[str] = None
    attributes: Optional[List[Attribute]] = None
    tags: Optional[List[str]] = None
    buying_price: Optional[float] = None
    dimensions: Optional[Dimensions] = None
    category: Optional[Union[str, MeliCategory]]
    brand: Optional[str] = None
    notes: Optional[str] = None
    deals: Optional[List[Deal]] = None
    stocks: Optional[List[Stock]] = None
    prices: Optional[List[Price]] = None
    pictures: Optional[List[Picture]] = None


class Shipping(BaseModel):
    local_pickup: Optional[bool] = Field(default=None, alias='localPickup')
    mode: Optional[str] = None
    free_shipping: Optional[bool] = Field(default=None, alias='freeShipping')
    free_shipping_cost: Optional[float] = Field(default=None, alias='freeShippingCost')
    mandatory_free_shipping: Optional[bool] = Field(default=None, alias='mandatoryFreeShipping')
    free_shipping_method: Optional[str] = Field(default=None, alias='freeShippingMethod')


class MShopsShipping(BaseModel):
    enabled: Optional[bool] = None


class AttributeCompletion(BaseModel):
    product_identifier_status: Optional[str] = Field(default=None, alias='productIdentifierStatus')
    data_sheet_status: Optional[str] = Field(default=None, alias='dataSheetStatus')
    status: Optional[str] = None
    count: Optional[int] = None
    total: Optional[int] = None


class MeliProduct(Product):
    product_id: Optional[int] = Field(default=None, alias='productId')
    has_custom_shipping_costs: Optional[bool] = Field(default=None, alias='hasCustomShippingCosts')
    shipping: Optional[Shipping] = None
    mshops_shipping: Optional[MShopsShipping] = Field(default=None, alias='mShopsShipping')
    add_free_shipping_cost_to_price: Optional[bool] = Field(default=None, alias='addFreeShippingCostToPrice')
    category: Optional[Union[str, MeliCategory]]  # will never be str, but needs compat with super class
    attribute_completion: Optional[AttributeCompletion] = Field(default=None, alias='attributeCompletion')
    catalog_products: Optional[List[str]] = Field(default=None, alias='catalogProducts')
    warranty: Optional[str] = None
    domain: Optional[str] = None
    listing_type_id: Optional[str] = Field(default=None, alias='listingTypeId')
    catalog_products_status: Optional[str] = Field(default=None, alias='catalogProductsStatus')


@dataclass
class ProductService(BaseService[Product]):
    endpoint: str = Field(default='products', exclude=True)
    create_if_it_doesnt_exist: bool = Field(default=False, exclude=True)

    def __call__(self, **payload):
        self._record = Product(**payload)
        return self

    def create(self):
        endpoint_url = self.config.get_endpoint(f'{self.endpoint}/synchronize')
        headers = self.config.headers.copy()
        headers.update({"createifitdoesntexist": str(self.create_if_it_doesnt_exist).lower()})
        data = self._record.model_dump_json(by_alias=True, exclude_none=True)
        response = requests.post(endpoint_url, data=data, headers=headers)
        if response.status_code == 204:
            raise Exception("Product does not exist and the request cant create if it does not exist")
        return Product(**response.json())

    def update(self):
        # TODO: Change name to synchronize
        endpoint_url = self.config.get_endpoint(f'{self.endpoint}/synchronize')
        headers = self.config.headers.copy()
        data = self._record.model_dump_json(by_alias=True, exclude_none=True)
        if not self._record.code and not self._record.sku:
            raise "Sku or code should be provided to update the product"
        response = requests.post(endpoint_url, data=data, headers=headers)
        if response.status_code == 204:
            raise Exception("Product does not exist and the request cant create if it does not exist")
        return Product(**response.json())

    def get(self, product_id: int) -> "Product":
        endpoint_url = self.config.get_endpoint(f'{self.endpoint}/{product_id}')
        headers = self.config.headers
        response = requests.get(endpoint_url, headers=headers)
        if not response.ok:
            raise Exception(f"Error getting product {product_id}\n {response.text}")
        response_data = response.json()
        return Product(**response_data)

    def get_bundle(self, product_id: int) -> "Product":
        endpoint_url = self.config.get_endpoint(f'{self.endpoint}/{product_id}/bundles')
        headers = self.config.headers
        response = requests.get(endpoint_url, headers=headers)
        if not response.ok:
            raise Exception(f"Error getting bundle {product_id}\n {response.text}")
        return Product(**response.json())

    def get_ml_integration(self, product_id: int) -> "MeliProduct":
        endpoint_url = self.config.get_endpoint(f'{self.endpoint}/{product_id}/listingintegration')
        headers = self.config.headers
        response = requests.get(endpoint_url, headers=headers)
        if not response.ok:
            raise Exception(f"Error getting ml integration {product_id}\n {response.text}")
        return MeliProduct(**response.json())

    def search(self, params: SearchProductParams) -> SearchProduct:
        endpoint: str = f'search/{self.endpoint}'
        headers = self.config.headers
        url = self.config.get_endpoint(endpoint)
        response = requests.get(url, headers=headers, params=params.model_dump(by_alias=True, exclude_none=True))
        return SearchProduct(**response.json())



from pydantic import BaseModel, Field
from typing import List, Optional
import requests
from producteca.abstract.abstract_dataclass import BaseService
from producteca.sales_orders.search_sale_orders import SearchSalesOrderParams, SearchSalesOrder
from producteca.payments.payments import Payment
from producteca.shipments.shipment import Shipment
from dataclasses import dataclass
import logging
_logger = logging.getLogger(__name__)


class SaleOrderLocation(BaseModel):
    streetName: Optional[str] = None
    streetNumber: Optional[str] = None
    addressNotes: Optional[str] = None
    state: Optional[str] = None
    city: Optional[str] = None
    neighborhood: Optional[str] = None
    zipCode: Optional[str] = None


class SaleOrderBillingInfo(BaseModel):
    docType: Optional[str] = None
    docNumber: Optional[str] = None
    streetName: Optional[str] = None
    streetNumber: Optional[str] = None
    comment: Optional[str] = None
    zipCode: Optional[str] = None
    city: Optional[str] = None
    state: Optional[str] = None
    stateRegistration: Optional[str] = None
    taxPayerType: Optional[str] = None
    firstName: Optional[str] = None
    lastName: Optional[str] = None
    businessName: Optional[str] = None


class SaleOrderProfile(BaseModel):
    app: int
    integrationId: str
    nickname: Optional[str] = None


class SaleOrderContact(BaseModel):
    id: int
    name: str
    contactPerson: Optional[str] = None
    mail: Optional[str] = None
    phoneNumber: Optional[str] = None
    taxId: Optional[str] = None
    location: Optional[SaleOrderLocation] = None
    notes: Optional[str] = None
    type: Optional[str] = None
    priceList: Optional[str] = None
    priceListId: Optional[str] = None
    profile: Optional[SaleOrderProfile] = None
    billingInfo: Optional[SaleOrderBillingInfo] = None


class SaleOrderIntegrationId(BaseModel):
    alternateId: Optional[str] = None
    integrationId: str
    app: int


class SaleOrderVariationPicture(BaseModel):
    url: str
    id: Optional[int] = None


class SaleOrderVariationStock(BaseModel):
    warehouseId: Optional[int] = None
    warehouse: str
    quantity: int
    reserved: int
    lastModified: Optional[str] = None
    available: int


class SaleOrderVariationAttribute(BaseModel):
    key: str
    value: str


class SaleOrderVariation(BaseModel):
    supplierCode: Optional[str] = None
    pictures: Optional[List[SaleOrderVariationPicture]] = None
    stocks: Optional[List[SaleOrderVariationStock]] = None
    integrationId: Optional[int] = None
    attributesHash: Optional[str] = None
    primaryColor: Optional[str] = None
    secondaryColor: Optional[str] = None
    size: Optional[str] = None
    thumbnail: Optional[str] = None
    attributes: Optional[List[SaleOrderVariationAttribute]] = None
    integrations: Optional[List[SaleOrderIntegrationId]] = None
    id: int
    sku: str
    barcode: Optional[str] = None


class SaleOrderProduct(BaseModel):
    name: str
    code: str
    brand: Optional[str] = None
    id: int


class SaleOrderConversation(BaseModel):
    questions: Optional[List[str]] = None


class SaleOrderLine(BaseModel):
    price: float
    originalPrice: Optional[float] = None
    transactionFee: Optional[float] = None
    product: SaleOrderProduct
    variation: SaleOrderVariation
    orderVariationIntegrationId: Optional[str] = None
    quantity: int
    conversation: Optional[SaleOrderConversation] = None
    reserved: Optional[int] = None
    id: int


class SaleOrderCard(BaseModel):
    paymentNetwork: Optional[str] = None
    firstSixDigits: Optional[int] = None
    lastFourDigits: Optional[int] = None
    cardholderIdentificationNumber: Optional[str] = None
    cardholderIdentificationType: Optional[str] = None
    cardholderName: Optional[str] = None


class SaleOrderPaymentIntegration(BaseModel):
    integrationId: str
    app: int


class SaleOrderPayment(BaseModel):
    date: Optional[str] = None
    amount: float
    couponAmount: Optional[float] = None
    status: Optional[str] = None
    method: Optional[str] = None
    integration: Optional[SaleOrderPaymentIntegration] = None
    transactionFee: Optional[float] = None
    installments: Optional[int] = None
    card: Optional[SaleOrderCard] = None
    notes: Optional[str] = None
    authorizationCode: Optional[str] = None
    hasCancelableStatus: Optional[bool] = None
    id: Optional[int] = None


class SaleOrderShipmentMethod(BaseModel):
    trackingNumber: Optional[str] = None
    trackingUrl: Optional[str] = None
    courier: Optional[str] = None
    mode: Optional[str] = None
    cost: Optional[float] = None
    type: Optional[str] = None
    eta: Optional[int] = None
    status: Optional[str] = None


class SaleOrderShipmentProduct(BaseModel):
    product: int
    variation: int
    quantity: int


class SaleOrderShipmentIntegration(BaseModel):
    app: int
    integrationId: str
    status: str
    id: int


class SaleOrderShipment(BaseModel):
    date: str
    products: List[SaleOrderShipmentProduct]
    method: Optional[SaleOrderShipmentMethod] = None
    integration: Optional[SaleOrderShipmentIntegration] = None
    receiver: Optional[dict] = None
    id: int


class SaleOrderInvoiceIntegration(BaseModel):
    id: Optional[int] = None
    integrationId: Optional[str] = None
    app: Optional[int] = None
    createdAt: Optional[str] = None
    documentUrl: Optional[str] = None
    xmlUrl: Optional[str] = None
    decreaseStock: Optional[bool] = None


class SaleOrder(BaseModel):
    tags: Optional[List[str]] = None
    integrations: Optional[List[SaleOrderIntegrationId]] = None
    invoiceIntegration: Optional[SaleOrderInvoiceIntegration] = None
    channel: Optional[str] = None
    piiExpired: Optional[bool] = None
    contact: Optional[SaleOrderContact] = None
    lines: Optional[List[SaleOrderLine]] = None
    warehouse: Optional[str] = None
    warehouseId: Optional[int] = None
    warehouseIntegration: Optional[str] = None
    pickUpStore: Optional[str] = None
    payments: Optional[List[SaleOrderPayment]] = None
    shipments: Optional[List[SaleOrderShipment]] = None
    amount: Optional[float] = None
    shippingCost: Optional[float] = None
    financialCost: Optional[float] = None
    paidApproved: Optional[float] = None
    paymentStatus: Optional[str] = None
    deliveryStatus: Optional[str] = None
    paymentFulfillmentStatus: Optional[str] = None
    deliveryFulfillmentStatus: Optional[str] = None
    deliveryMethod: Optional[str] = None
    paymentTerm: Optional[str] = None
    currency: Optional[str] = None
    customId: Optional[str] = None
    isOpen: Optional[bool] = None
    isCanceled: Optional[bool] = None
    cartId: Optional[str] = None
    draft: Optional[bool] = None
    promiseDeliveryDate: Optional[str] = None
    promiseDispatchDate: Optional[str] = None
    hasAnyShipments: Optional[bool] = None
    hasAnyPayments: Optional[bool] = None
    date: Optional[str] = None
    notes: Optional[str] = None
    id: int


@dataclass
class SaleOrderService(BaseService[SaleOrder]):
    endpoint: str = Field(default='salesorders', exclude=True)

    def __call__(self, **payload):
        self._record = SaleOrder(**payload)
        return self

    def get(self, sale_order_id: int) -> "SaleOrder":
        endpoint = f'{self.endpoint}/{sale_order_id}'
        url = self.config.get_endpoint(endpoint)
        response = requests.get(url, headers=self.config.headers)
        if not response.ok:
            raise Exception("Order could not be fetched")
        return SaleOrder(**response.json())

    def get_shipping_labels(self):
        if not self._record:
            raise Exception("You need to add a record id")
        endpoint = f'{self.endpoint}/{self._record.id}/labels'
        url = self.config.get_endpoint(endpoint)
        response = requests.get(url, headers=self.config.headers)
        if not response.ok:
            raise Exception("labels could not be gotten")
        return response.json()

    def close(self):
        if not self._record:
            raise Exception("You need to add a record id")
        endpoint = f'{self.endpoint}/{self._record.id}/close'
        url = self.config.get_endpoint(endpoint)
        response = requests.post(url, headers=self.config.headers)
        if not response.ok:
            raise Exception("Order could not be closed")

    def cancel(self):
        if not self._record:
            raise Exception("You need to add a record id")
        endpoint = f'{self.endpoint}/{self._record.id}/cancel'
        url = self.config.get_endpoint(endpoint)
        response = requests.post(url, headers=self.config.headers)
        if not response.ok:
            raise Exception("Order could not be closed")

    def synchronize(self, payload: "SaleOrder") -> "SaleOrder":
        endpoint = f'{self.endpoint}/synchronize'
        url = self.config.get_endpoint(endpoint)
        response = requests.post(url, data=payload.model_dump_json(exclude_none=True), headers=self.config.headers)
        if not response.ok:
            raise Exception(f"Synchronize error {response.text}")
        return SaleOrder(**response.json())

    def invoice_integration(self):
        if not self._record:
            raise Exception("You need to add a record id")
        endpoint = f'{self.endpoint}/{self._record.id}/invoiceIntegration'
        url = self.config.get_endpoint(endpoint)
        response = requests.put(url, headers=self.config.headers, data=self._record.model_dump_json(exclude_none=True))
        if not response.ok:
            raise Exception(f"Error on resposne {response.text}")
        return SaleOrder(**response.json())

    def search(self, params: SearchSalesOrderParams):
        endpoint: str = f"search/{self.endpoint}"
        headers = self.config.headers
        url = self.config.get_endpoint(endpoint)
        new_url = f"{url}?$filter={params.filter}&top={params.top}&skip={params.skip}"
        response = requests.get(
            new_url,
            headers=headers,
        )
        if not response.ok:
            raise Exception(f"Error on resposne {response.text}")
        return SearchSalesOrder(**response.json())

    def add_payment(self, payload: "Payment") -> "Payment":
        if not self._record:
            raise Exception("You need to add a record id")
        url = self.config.get_endpoint(f"{self.endpoint}/{self._record.id}/payments")
        res = requests.post(url, data=payload.model_dump_json(exclude_none=True), headers=self.config.headers)
        if not res.ok:
            raise Exception(f"Error on resposne {res.text}")
        return Payment(**res.json())

    def update_payment(self, payment_id: int, payload: "Payment") -> "Payment":
        if not self._record:
            raise Exception("You need to add a record id")
        url = self.config.get_endpoint(f"{self.endpoint}/{self._record.id}/payments/{payment_id}")
        res = requests.put(url, data=payload.model_dump_json(exclude_none=True), headers=self.config.headers)
        if not res.ok:
            raise Exception(f"Error on payment update {res.text}")
        return Payment(**res.json())

    def add_shipment(self, payload: "Shipment") -> "Shipment":
        if not self._record:
            raise Exception("You need to add a record id")
        url = self.config.get_endpoint(f"{self.endpoint}/{self._record.id}/shipments")
        res = requests.post(url, data=payload.model_dump_json(exclude_none=True), headers=self.config.headers)
        if not res.ok:
            raise Exception(f"Error on shipment add {res.text}")
        return Shipment(**res.json())

    def update_shipment(self, shipment_id: str, payload: "Shipment") -> "Shipment":
        if not self._record:
            raise Exception("You need to add a record id")
        url = self.config.get_endpoint(f"{self.endpoint}/{self._record.id}/shipments/{shipment_id}")
        res = requests.put(url, data=payload.model_dump_json(exclude_none=True), headers=self.config.headers)
        if not res.ok:
            raise Exception(f"Error on shipment update {res.text}")
        return Shipment(**res.json())



import unittest
import json
from unittest.mock import patch, Mock
from producteca.sales_orders.search_sale_orders import SearchSalesOrderParams, SalesOrderResultItem
from producteca.client import ProductecaClient


class TestSearchSalesOrder(unittest.TestCase):

    def setUp(self):
        self.client = ProductecaClient(token="test_client_id", api_key="test_client_secret")
        self.params = SearchSalesOrderParams(
            top=10,
            skip=0,
            filter="status eq 'confirmed'"
        )

    @patch('requests.get')
    def test_search_saleorder_success(self, mock_get):
        # Mock successful response
        mock_response = Mock()
        with open('producteca/sales_orders/tests/search.json', 'r') as f:
            results = json.loads(f.read())
            mock_response.json.return_value = results
            mock_response.status_code = 200
            mock_get.return_value = mock_response

        response = self.client.SalesOrder.search(self.params)
        
        # Validate response
        self.assertEqual(response.count, 0)
        self.assertEqual(len(response.results), 1)
        self.assertEqual(response.results[0].id, "string")
        self.assertIsInstance(response.results[0], SalesOrderResultItem)

    @patch('requests.get')
    def test_search_saleorder_error(self, mock_get):
        # Mock error response
        mock_response = Mock()
        mock_response.json.return_value = {"error": "Invalid request"}
        mock_response.status_code = 400
        mock_get.return_value = mock_response
        with self.assertRaises(Exception):
            self.client.SalesOrder.search(self.params)
       

if __name__ == '__main__':
    unittest.main()



from producteca.config.config import ConfigProducteca
from producteca.products.products import ProductService
from producteca.sales_orders.sales_orders import SaleOrderService
import os


class ProductecaClient:

    def __init__(self, token: str = os.environ.get('PRODUCTECA_TOKEN', ''), api_key: str = os.environ.get('PRODUCTECA_API_KEY', '')):
        if not token:
            raise ValueError('PRODUCTECA_TOKEN environment variable not set')
        if not api_key:
            raise ValueError('PRODUCTECA_API_KEY environment variable not set')
        self.config = ConfigProducteca(token=token, api_key=api_key)

    @property
    def Product(self):
        return ProductService(self.config)

    @property
    def SalesOrder(self):
        return SaleOrderService(self.config)



from typing import List, Optional
from pydantic import BaseModel


class ShipmentProduct(BaseModel):
    product: int
    variation: Optional[int] = None
    quantity: int


class ShipmentMethod(BaseModel):
    trackingNumber: Optional[str] = None
    trackingUrl: Optional[str] = None
    courier: Optional[str] = None
    mode: Optional[str] = None
    cost: Optional[float] = None
    type: Optional[str] = None
    eta: Optional[int] = None
    status: Optional[str] = None


class ShipmentIntegration(BaseModel):
    id: Optional[int] = None
    integrationId: Optional[str] = None
    app: Optional[int] = None
    status: str


class Shipment(BaseModel):
    date: Optional[str] = None
    products: Optional[List[ShipmentProduct]] = None
    method: Optional[ShipmentMethod] = None
    integration: Optional[ShipmentIntegration] = None



from abc import ABC
from ..config.config import ConfigProducteca
from dataclasses import dataclass
from typing import Optional


@dataclass
class BaseService[T](ABC):
    config: ConfigProducteca
    endpoint: str
    _record: Optional[T] = None


import unittest
from unittest.mock import patch, Mock
from producteca.products.products import Product
from producteca.client import ProductecaClient


class TestProduct(unittest.TestCase):
    def setUp(self):
        self.client = ProductecaClient(token="test_client_id", api_key="test_client_secret")
        self.test_product = Product(
            sku="TEST001",
            name="Test Product",
            code="TEST001",
            category="Test"
        )

    @patch('requests.post')
    def test_create_product_success(self, mock_post):
        # Mock successful response
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = self.test_product.model_dump()
        mock_post.return_value = mock_response

        response = self.client.Product(**self.test_product.model_dump()).create()
        
        self.assertEqual(response.sku, "TEST001")

    @patch('requests.post')
    def test_create_product_not_exist(self, mock_post):
        # Mock product not found response
        mock_response = Mock()
        mock_response.status_code = 204
        mock_post.return_value = mock_response

        with self.assertRaises(Exception):
            self.client.Product.create()

    @patch('requests.post')
    def test_update_product_success(self, mock_post):
        # Mock successful update
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = self.test_product.model_dump()
        mock_post.return_value = mock_response

        response = self.client.Product(**self.test_product.model_dump()).update()
        
        self.assertEqual(response.name, "Test Product")

    @patch('requests.get')
    def test_get_product(self, mock_get):
        # Mock get product response
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = self.test_product.model_dump()
        mock_get.return_value = mock_response

        response = self.client.Product.get(1)
        
        self.assertEqual(response.sku, "TEST001")

    @patch('requests.get')
    def test_get_bundle(self, mock_get):
        # Mock get bundle response
        mock_response = Mock()
        mock_response.status_code = 200
        test_prod = self.test_product.model_dump()
        test_prod.update({"sku": "TEST001", "bundles": []})
        mock_response.json.return_value = test_prod
        mock_get.return_value = mock_response

        product = self.client.Product.get_bundle(1)
        
        self.assertEqual(product.sku, "TEST001")

    @patch('requests.get')
    def test_get_ml_integration(self, mock_get):
        # Mock ML integration response
        mock_response = Mock()
        mock_response.status_code = 200
        test_prod = self.test_product.model_dump()
        test_prod.update({"sku": "TEST001", "integrations": []})
        mock_response.json.return_value = test_prod
        mock_get.return_value = mock_response

        product = self.client.Product.get_ml_integration(1)
        
        self.assertEqual(product.sku, "TEST001")


if __name__ == '__main__':
    unittest.main()
